<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://upcasted.com
 * @since      1.0.0
 *
 * @package    Ai_For_Wordpress_By_Upcasted
 * @subpackage Ai_For_Wordpress_By_Upcasted/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Ai_For_Wordpress_By_Upcasted
 * @subpackage Ai_For_Wordpress_By_Upcasted/admin
 * @author     Upcasted <contact@upcasted.com>
 */
class Ai_For_Wordpress_By_Upcasted_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /** @var UpcastedEncryption $encryption */
    private $encryption;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->encryption = UpcastedEncryption::getInstance();
    }

    function add_admin_page()
    {
        add_submenu_page(
            'upload.php',
            __('AI Image Generator', 'ai-wp-upcasted'),
            __('AI Generator', 'ai-wp-upcasted'),
            'manage_options',
            'ai-wp-upcasted',
            array($this, 'main_app')
        );

        add_options_page(
            __('AI Image Generator Settings', 'ai-wp-upcasted'),
            __('AI Image Generator', 'ai-wp-upcasted'),
            'manage_options',
            'ai-wp-upcasted-options',
            array($this, 'settings_page')
        );
    }

    function main_app()
    {
?>
        <div id="ai-wp-upcasted-main-app"></div>
    <?php
    }

    function set_api_key()
    {
        check_ajax_referer('_wpnonce', 'security');

        if (!current_user_can('manage_options')) {
            return new WP_Error('unauthorized', __('You are not authorized to perform this action.', 'ai-wp-upcasted'));
        }

        if (isset($_POST['ai_wp_upcasted_api_key']) && !empty($_POST['ai_wp_upcasted_api_key'])) {
            $apiKey = sanitize_text_field($_POST['ai_wp_upcasted_api_key']);

            $image_controller = new ImageController();
            $apiKeyIsValid = $image_controller->validate_api_key($apiKey);

            if (isset($apiKeyIsValid['error'])) {
                $error = new WP_Error($apiKeyIsValid['error']['code'], $apiKeyIsValid['error']['message']);

                wp_send_json_error($error);
            }
            update_option('ai_wp_upcasted_api_key', UpcastedEncryption::getInstance()->encrypt($apiKey));
            wp_send_json_success();
        } else {
            $error = new WP_Error('Error', __('API key cannot be empty', 'ai-wp-upcasted'));
            wp_send_json_error($error);
        }
    }

    function set_image_variations_settings()
    {
        check_ajax_referer('_wpnonce', 'security');

        if (!current_user_can('manage_options')) {
            return new WP_Error('unauthorized', __('You are not authorized to perform this action.', 'ai-wp-upcasted'));
        }

        $errors = new WP_Error();
        $response = array('success' => false);

        if (isset($_POST['variationsResolution'])) {
            $resolution = sanitize_text_field($_POST['variationsResolution']);
            if (!in_array($resolution, array('256x256', '512x512', '1024x1024'))) {
                $errors->add('invalid_resolution', __('Invalid image resolution selected.', 'ai-wp-upcasted'));
            } else {
                update_option('ai_wp_upcasted_resolution', $resolution);
            }
        }

        if (isset($_POST['noOfVariations'])) {
            $no_of_variations = intval($_POST['noOfVariations']);
            if ($no_of_variations < 1 || $no_of_variations > 10) {
                $errors->add('invalid_variations_number', __('Invalid number of image variations selected.', 'ai-wp-upcasted'));
            } else {
                update_option('ai_wp_upcasted_variations_num', $no_of_variations);
            }
        }

        if (is_wp_error($errors) && count($errors->get_error_codes()) > 0) {
            $response['data'] = array(
                'success' => false,
                'message' => $errors->get_error_message()
            );
        } else {
            $response['success'] = true;
            $response['data'] = array(
                'success' => true,
                'message' => __('Image variations settings saved successfully.')
            );
        }

        wp_send_json($response);
    }

    // Function to display the API Key input field in the OpenAI Image Generation page
    function settings_page()
    {
        if (!current_user_can('manage_options')) {
            return;
        }

        if (isset($_POST['ai_wp_upcasted_api_key']) && !empty($_POST['ai_wp_upcasted_api_key'])) {
            update_option(
                'ai_wp_upcasted_api_key',
                UpcastedEncryption::getInstance()->encrypt(
                    sanitize_text_field(
                        $_POST['ai_wp_upcasted_api_key']
                    )
                )
            );
        }

        if (isset($_POST['ai_wp_upcasted_resolution'])) {
            update_option(
                'ai_wp_upcasted_resolution',
                sanitize_text_field(
                    $_POST['ai_wp_upcasted_resolution']
                )
            );
        }

        if (isset($_POST['ai_wp_upcasted_variations_num'])) {
            update_option(
                'ai_wp_upcasted_variations_num',
                sanitize_text_field(
                    intval($_POST['ai_wp_upcasted_variations_num'])
                )
            );
        }

        $api_key = get_option('ai_wp_upcasted_api_key', '');
        $selected_option = get_option('ai_wp_upcasted_resolution');
        $variations_num = get_option('ai_wp_upcasted_variations_num');
    ?>
        <div class="wrap">
            <form action="" method="post">
                <table class="form-table">
                    <tr>
                        <th scope="row">
                            <label for="ai_wp_upcasted_api_key">OpenAI Secret API Key</label><br />

                        </th>
                        <td>
                            <?php if (empty($api_key)) { ?>
                                <input type="password" id="ai_wp_upcasted_api_key" name="ai_wp_upcasted_api_key" value="" placeholder="Write here your new OpenAI API Key" class="regular-text">
                                <a href="https://help.openai.com/en/articles/4936850-where-do-i-find-my-secret-api-key" target="_blank">
                                    Where do I find my Secret API Key?
                                </a>
                                <p>To get an API key you will need to create an account on <a href="https://openai.com/api/" target="_blank">OpenAI.com</a></p>
                            <?php } else { ?>
                                <strong style="color:green">
                                    <span class="dashicons dashicons-saved"></span>
                                    OpenAI API Key set and ready!
                                </strong>
                                <a href="#" onclick="document.getElementById('upcasted-new-api-key').style.display = 
                                                        document.getElementById('upcasted-new-api-key').style.display === 'none' ? 'block' : 'none'">
                                    Change API key
                                </a>
                                <br /><br />
                                <div id="upcasted-new-api-key" style="display: none">
                                    <span>Write here your new OpenAI API Key if you need to change it:</span><br />
                                    <input type="password" id="ai_wp_upcasted_api_key" name="ai_wp_upcasted_api_key" value="" placeholder="OpenAI API Key" class="regular-text"><br /><br />
                                    <a href="https://help.openai.com/en/articles/4936850-where-do-i-find-my-secret-api-key" target="_blank">
                                        Where do I find my Secret API Key?
                                    </a>
                                    <br />
                                    <p>To get an API key you will need to create an account on <a href="https://openai.com/api/" target="_blank">OpenAI.com</a></p>
                                </div>
                            <?php } ?>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="ai_wp_upcasted_resolution">Resolution (pixels)</label>
                        </th>
                        <td>
                            <?php
                            $options = array(
                                '256x256' => '256x256',
                                '512x512' => '512x512',
                                '1024x1024' => '1024x1024'
                            );
                            ?>
                            <select name="ai_wp_upcasted_resolution">';
                                <?php
                                foreach ($options as $value => $label) {
                                    $selected = ($selected_option === $value) ? 'selected' : '';
                                ?>
                                    <option value="<?php echo esc_html($value) ?>" <?php echo esc_html($selected) ?>>
                                        <?php echo esc_html($label) ?>
                                    </option>
                                <?php
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="ai_wp_upcasted_variations_num">No. of variations</label>
                        </th>
                        <td>
                            <input type="number" id="ai_wp_upcasted_variations_num" name="ai_wp_upcasted_variations_num" value="<?php echo esc_attr($variations_num); ?>" min="1" max="10" class="regular-text">
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
<?php
    }

    /**
     * Register the plugin settings.
     */
    public function register_settings()
    {
        register_setting(
            'ai_wp_upcasted',
            'ai_wp_upcasted_settings',
            array($this, 'sanitize_options')
        );

        //     add_settings_section(
        //         'ai_wp_upcasted_settings',
        //         __('OpenAI Image Generation Settings', 'ai-wp-upcasted'),
        //         array($this, 'render_settings_section'),
        //         'ai-wp-upcasted-options'
        //     );

        //     add_settings_field(
        //         'ai_wp_upcasted_api_key',
        //         __('API Key', 'ai-wp-upcasted'),
        //         array($this, 'render_api_key_setting'),
        //         'ai-wp-upcasted-options',
        //         'ai_wp_upcasted_settings'
        //     );

        //     add_settings_field(
        //         'ai_wp_upcasted_resolution',
        //         __('Resolution', 'ai-wp-upcasted'),
        //         array($this, 'render_resolution_setting'),
        //         'ai-wp-upcasted-options',
        //         'ai_wp_upcasted_settings'
        //     );

        //     add_settings_field(
        //         'ai_wp_upcasted_variations_num',
        //         __('No. of variations', 'ai-wp-upcasted'),
        //         array($this, 'render_variations_num_setting'),
        //         'ai-wp-upcasted-options',
        //         'ai_wp_upcasted_settings'
        //     );
    }

    /**
     * Sanitize the plugin options.
     *
     * @param array $input The raw input data.
     * @return array The sanitized input data.
     */
    public function sanitize_options($input)
    {
        $output = array();

        if (isset($input['ai_wp_upcasted_api_key'])) {
            $output['ai_wp_upcasted_api_key'] = sanitize_text_field($input['ai_wp_upcasted_api_key']);
        }

        if (isset($input['ai_wp_upcasted_resolution'])) {
            $output['ai_wp_upcasted_resolution'] = sanitize_text_field($input['ai_wp_upcasted_resolution']);
        }

        if (isset($input['ai_wp_upcasted_variations_num'])) {
            $output['ai_wp_upcasted_variations_num'] = sanitize_text_field(intval($input['ai_wp_upcasted_variations_num']));
        }

        return $output;
    }

    /**
     * Render the settings section.
     */
    // public function render_settings_section()
    // {
    //     echo '<p>' . __('Configure the OpenAI Image Generation API settings', 'ai-wp-upcasted') . '</p>';
    // }

    /**
     * Render the API key setting.
     */
    // public function render_api_key_setting()
    // {
    //     $option = get_option('ai_wp_upcasted_api_key');
    //     echo '<input type="password" name="ai_wp_upcasted_options[ai_wp_upcasted_api_key]" value="' . (!empty($option) ? esc_html($option) : '') . '" class="regular-text" />';
    // }

    /**
     * Render the resolution setting.
     */
    // public function render_resolution_setting()
    // {
    //     $options = array(
    //         '256x256' => '256x256',
    //         '512x512' => '512x512',
    //         '1024x1024' => '1024x1024'
    //     );
    //     $selected_option = get_option('ai_wp_upcasted_resolution');
    //     echo '<select name="ai_wp_upcasted_options[ai_wp_upcasted_resolution]">';
    //     foreach ($options as $value => $label) {
    //         $selected = ($selected_option === $value) ? 'selected' : '';
    //         echo '<option value="' . esc_attr($value) . '" ' . esc_attr($selected) . '>' . esc_html($label) . '</option>';
    //     }
    //     echo '</select>';
    // }

    /**
     * Render the number of images setting.
     */
    // public function render_variations_num_setting()
    // {
    //     $option = get_option('ai_wp_upcasted_variations_num');
    //     echo '<input type="number" name="ai_wp_upcasted_options[ai_wp_upcasted_variations_num]" value="' . (!empty($option) ? esc_attr($option) : '') . '" class="small-text" min="1" />';
    // }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Ai_For_Wordpress_By_Upcasted_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Ai_For_Wordpress_By_Upcasted_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/ai-wp-upcasted-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Ai_For_Wordpress_By_Upcasted_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Ai_For_Wordpress_By_Upcasted_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        $currentScreen = get_current_screen();

        wp_enqueue_media();
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/ai-wp-upcasted-admin.js', array('jquery'), $this->version, false);

        if ($currentScreen->base == 'media_page_ai-wp-upcasted') {
            $pluginDirUrl = plugin_dir_url(__DIR__);
            $noOfVariations = get_option('ai_wp_upcasted_variations_num', '');
            $variationsResolution = get_option('ai_wp_upcasted_resolution', '');
            $apiIsSet = get_option('ai_wp_upcasted_api_key', '');

            wp_enqueue_script('ai-wp-upcasted-react', plugin_dir_url(__DIR__) . 'dist/ai-wp-upcasted-react.js', array('wp-element', 'wp-api'), $this->version, false);
            wp_localize_script(
                'ai-wp-upcasted-react',
                'ai_wp_upcasted_object',
                array(
                    'noOfVariations' => $noOfVariations,
                    'variationsResolution' => $variationsResolution,
                    'apiIsSet' => !empty($apiIsSet) ? true : false,
                    'security' => wp_create_nonce('_wpnonce'),
                    'pluginDirUrl' => $pluginDirUrl,
                )
            );
        }

        if ($currentScreen->id === 'upload') {
            wp_enqueue_script('ai-wp-upcasted-media-library', plugin_dir_url(__DIR__) . 'admin/js/ai-wp-upcasted-media-library.js', array('jquery'), $this->version, true);
            wp_localize_script(
                'ai-wp-upcasted-media-library',
                'upcasted_media_library',
                array(
                    'buttonTitle' => __('Generate with AI', 'ai-wp-upcasted'),
                )
            );
        }
    }
}
